
//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.


#ifndef ALIGNMENT_H
#define ALIGNMENT_H

#include <QString>
#include <vector>
#include <memory>
#include "alignmentpoint.h"
#include "gat-core_global.h"
#include <QRectF>
#include "profilemeasurement.h"

class GATCORESHARED_EXPORT Alignment
{
public:
    //Points in this context are 2-D and have no description.
    //Point number is the index of QVector, thus it is also 0 based counting.
    Alignment();
    Alignment(const QString &);
    Alignment(const Alignment &copyAlignment);
    void copyAlignment(const Alignment & al);
    void setId(const int);
    void setDescription(const QString &);
    //void setPoint(const int pn, Point2D & point);
    void setPoint(const int pn, const double n, const double e);
    void setNorthing(const int pn, const double value);
    void setEasting(const int pn, const double value);
    void setStationing(const int pn, const double value);
    void setPointType(const int pn, const AlignmentPoint::PointType type);
    bool setCenterPoint(const int pn, const double easting, const double northing);
    bool setCenterNorthing(const int pn, const double northing);
    bool setCenterEasting(const int pn, const double easting);
    double getStationing(const int pn) const;
    double getEasting(const int pn) const;
    double getNorthing(const int pn) const;
    AlignmentPoint::PointType getPointType(const int pn);
    QPointF getCenterPoint(const int pn);
    void addPoint(AlignmentPoint &point);
    void insertPoint(const int pn);
    bool removePoints(const int pnStart, const int count);
    bool isPointValid(const int) const;
    int getId() const;
    QString getDescription()const;
    AlignmentPoint getPoint(const int) const;
    std::unique_ptr<AlignmentPoint> & getPointStdPtr(const int) const;
    //QGraphicsItem* getPointGraphicsItem(const int) const;
    QRectF boundingRect() const;
    int getPointCount() const;
    QPointF getPointF(const int pn);
//    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) override;
    //Todo: add alignment type for design PT/PC for centerline or PT/PT
	Alignment& operator=(const Alignment &);
    void calculateStationing();
    void setPointVector(std::unique_ptr<std::vector<std::unique_ptr<AlignmentPoint>>>);

    //measurement parameters
    void setMeasurementList(std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement> > >);
    int getMeasurementCount() const;

    qreal getPointOneX(const int pn) const;
    qreal getPointOneY(const int pn) const;
    qreal getPointTwoX(const int pn) const;
    qreal getPointTwoY(const int pn) const;
    int getPointType(const int pn) const;
    qreal getMeasurementValue(const int pn) const;
    int getMeasurementId(const int pn) const;

    bool removeMeasurements(const int row, const int count);
    void setMeasurementId(const int pn, const int id) const;

    void addMeasurementPoint(const QPointF &pointOne, const QPointF &pointTwo);
    void calculateBoundingRect();

private:
    std::unique_ptr<std::vector<std::unique_ptr<AlignmentPoint>>> mPointVector;
    int mId;
    QString mDescription;
    qreal mMinNorthing;
    qreal mMinEasting;
    qreal mMaxNorthing;
    qreal mMaxEasting;

    //This flag is to be set on all points to allow the point to communicate an update.
    QPointF getAlignmentPointF() const;
    qreal getSegmentLength(const int pn) const;

    std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement>>> mMeasurements;
    bool isMeasurementPointValid(const int pn)const;
};

#endif // ALIGNMENT_H
