//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef ALIGNMENTDAO_H
#define ALIGNMENTDAO_H

#include <QString>
#include <memory>
#include <vector>

#include "alignment.h"
#include "profilemeasurement.h"

class QSqlDatabase;

class AlignmentDao
{
public:
    AlignmentDao(QSqlDatabase&);
    void init() const;
    void addAlignment(std::unique_ptr<Alignment> &) const;
    void removeAlignment(const int id) const;
    void updateAlignment(const std::unique_ptr<Alignment> &) const;
    std::unique_ptr<std::vector<std::unique_ptr<Alignment>>> getAlignments() const;
    Alignment getAlignment(const int id) const;
    void updateAlignmentPoints(const std::unique_ptr<Alignment> &) const;
    void updateAlignmentPoint(const std::unique_ptr<AlignmentPoint> & point, const int alignId, const int pn)const;

    //Measurement related functions
    void addMeasurement(std::unique_ptr<Alignment> &alignment, const int i)const;
    bool removeMeasurements(const std::unique_ptr<Alignment> &alignment, int row, int count) const;

private:
    void removeAlignmentPoints(const int id) const;
    void addAlignmentPoints(const std::unique_ptr<Alignment> &) const;
    void addMeasurements(std::unique_ptr<Alignment> &alignment) const;
    QSqlDatabase& mDatabase;
    std::unique_ptr<std::vector<std::unique_ptr<AlignmentPoint>>> getPoints(const int alignmentId) const;
    std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement>>> getMeasurments(const int alignmentId) const;
    //std::unique_ptr<std::vector<AlignmentPoint*>> parseLineStringGeometry(QString &) const;
    //QString toWKT(const std::unique_ptr<Alignment> &) const;
};

#endif // ALIGNMENTDAO_H
