//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef ALIGNMENTMODEL_H
#define ALIGNMENTMODEL_H

#include <QAbstractListModel>
#include <QHash>
#include <vector>
#include <memory>
#include "alignment.h"
#include "gatdatamanager.h"
#include "gat-core_global.h"

class GATCORESHARED_EXPORT AlignmentModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum Roles{
        IdRole = Qt::UserRole + 1,
        NameRole,
        PointsRole
    };

    AlignmentModel(QObject* parent = nullptr);

    QModelIndex addAlignment(std::unique_ptr<Alignment> alignment);

    //bool updateAlignment(const QModelIndex & index);
    int rowCount(const QModelIndex&parent = QModelIndex())const override;
    int columnCount(const QModelIndex& parent = QModelIndex())const override;
    QVariant data(const QModelIndex& index, const int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole) override;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
    bool removeRows(int row, int count, const QModelIndex& parent = QModelIndex()) override;
    bool insertRow(int row, const QModelIndex & parent = QModelIndex());
    QHash<int, QByteArray> roleNames() const override;
    QModelIndex index(int row, int column, const QModelIndex & parent=QModelIndex()) const override;
    Qt::ItemFlags flags(const QModelIndex & index) const override;
    QGraphicsItem* getAlignmentGraphicsItem(const QModelIndex & index) const;
    //std::unique_ptr<Alignment>& getAlignment(const QModelIndex & index) const;
    bool isIndexValid(const QModelIndex & index) const;
    bool hasChildren(const QModelIndex& index) const override;
    QModelIndex parent(const QModelIndex& index) const override;
    //QRectF boundingRect(const QModelIndex& index)const;

public slots:
    void addAlignmentDes(const QString&);
    void databaseUpdated();
    void addAlignmentPoint(const double, const double, const QModelIndex & parent);

    //void deleteAlignmentPoint(const int, const std::unique_ptr<Alignment>&);
    void addMeasurement(const QModelIndex & parent, const QPointF &pointOne, const QPointF &pointTwo);

private:
    QVariant alignmentPointData(const QModelIndex &index) const;
    bool setAlignmentPointData(const QModelIndex& index, const QVariant &value, int role = Qt::EditRole);

    QVariant dataMeasurementParameters(const QModelIndex &index, const int role) const;
    bool removeRowsMeasurements(int row, int count, const QModelIndex &parent);
    GATDataManager & mDb;
    std::unique_ptr<std::vector<std::unique_ptr<Alignment>>> mAlignments;
};

#endif // ALIGNMENTMODEL_H
