//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "alignmentpoint.h"
#include <QDebug>
//#include <QPainter>
//#include <QGraphicsSceneMouseEvent>
//#include <QGraphicsScene>


AlignmentPoint::AlignmentPoint()
{
    mNorthing = -9;
    mEasting = -9;
    mStationing = -9;
    mPointType = AlignmentPoint::Line;
    mRadius = 5;
    mCenterEasting = -9;
    mCenterNorthing = -9;
    //setFlag(QGraphicsItem::ItemIsMovable);
    //setFlag(QGraphicsItem::ItemSendsScenePositionChanges);
    //setFlag(QGraphicsItem::ItemIgnoresTransformations);
    //setPos(getPointF());
}

AlignmentPoint::AlignmentPoint(const double n, const double e)
{
    mNorthing = n;
    mEasting = e;
    mStationing = -9;
    mCenterEasting = -9;
    mCenterNorthing = -9;
    mPointType = AlignmentPoint::Line;
    mRadius = 5;
    //setPos(getPointF());
}

//AlignmentPoint::AlignmentPoint(const AlignmentPoint &copyPoint)
//{
//    mNorthing = copyPoint.mNorthing;
//    mEasting = copyPoint.mEasting;
//    mStationing = copyPoint.mStationing;
//    mRadius = copyPoint.mRadius;
//    setPos(getPointF());
//    mPointType = copyPoint.mPointType;
//}

//void AlignmentPoint::setUpdateFlag(bool* alignmentUpdateFlag){
//    mUpdateAlignment = alignmentUpdateFlag;
//}

double AlignmentPoint::getNorthing()
{
    return mNorthing;
}

double AlignmentPoint::getEasting()
{
    return mEasting;
}

double AlignmentPoint::getStationing()
{
    return mStationing;
}

QPointF AlignmentPoint::getCenter()
{
    return QPointF(mCenterEasting, mCenterNorthing);
}

AlignmentPoint::PointType AlignmentPoint::getPointType()
{
    return mPointType;
}

void AlignmentPoint::setNorthing(const double n)
{
    mNorthing = n;
    //setPos(getPointF());
}

void AlignmentPoint::setEasting(const double e)
{
    mEasting = e;
    //setPos(getPointF());
}

void AlignmentPoint::setStationing(const double s)
{
    mStationing = s;
}

void AlignmentPoint::setCenter(const double centerEasting, const double centerNorthing)
{
    mCenterEasting = centerEasting;
    mCenterNorthing = centerNorthing;
}

void AlignmentPoint::setCenterNorthing(const double northing)
{
    mCenterNorthing = northing;
}

void AlignmentPoint::setCenterEasting(const double easting)
{
    mCenterEasting = easting;
}

void AlignmentPoint::setPointType(const AlignmentPoint::PointType pointType)
{
    mPointType = pointType;
}

QPointF AlignmentPoint::getPointF()
{
    return QPointF(mEasting, mNorthing);
}

//QRectF AlignmentPoint::boundingRect() const
//{

//    return QRectF(-mRadius, -mRadius, width(), height());
//}

//qreal AlignmentPoint::width() const
//{
//    return 2*mRadius;
//}


//qreal AlignmentPoint::height() const
//{

//    return 2*mRadius/this->scale();
//}


//void AlignmentPoint::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
//{
//    Q_UNUSED(widget);
//    Q_UNUSED(option);
//    double hScale = painter->transform().m11();
//    mRadius = 5/hScale;
//    QBrush brush(Qt::green);
//    QPen pen(Qt::yellow,1/hScale);
//    painter->setBrush(brush);
//    painter->setPen(pen);
//    painter->drawEllipse(this->boundingRect());
//}

//void AlignmentPoint::setPos(const QPointF &pos)
//{
//    QPointF point = mCoordinateTransform.toPlottingCoordinates(pos);
//    QGraphicsItem::setPos(point);
//}

//void AlignmentPoint::setMode(const int mode)
//{
//    mMode = mode;
//}

//QVariant Point2D::itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant &)
//{
//    return getPointF();
//}

//void AlignmentPoint::mousePressEvent(QGraphicsSceneMouseEvent *event)
//{
//    qDebug() << "Point2D::mousePressEvent";
//    if(event->button() == Qt::LeftButton)
//    {
//        if(event->modifiers()  == Qt::AltModifier || mMode > 0){
//            qDebug() << "Point2D::mousePressEvent Left + Alt";
//            setSelected(true);
//            this->update();
//        }

//    }
//}

//void AlignmentPoint::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
//{
//    //Mode 0 - do nothing
//    //Mode 1 - move point
//    //Mode 2 - delete point
//    if(event->button() == Qt::LeftButton)
//    {
//        if(event->modifiers()  == Qt::AltModifier || mMode == 1){
//            qDebug() << "Point2D::mousePressEvent Left + Alt";
//            QPointF pointLocation = mCoordinateTransform.toDataCoordinates(event->scenePos());
//            mEasting = pointLocation.x();
//            mNorthing = pointLocation.y();
//            setPos(getPointF());
//            qDebug() << "easting, northing: " << mEasting << ", " << mNorthing;
//            this->update(boundingRect());
//            //itemChange(ItemPositionChange,getPointF());
//            //this sets the flag on the alignment to true if the pointer has been set.
//            *mUpdateAlignment = true;
//            emit pointMoved(this);
//        }
//        if(mMode == 2){

//            *mUpdateAlignment = true;
//            qDebug() << "Point2D::mouseReleaseEvent emit deletePoint";
//            emit deletePoint(this);
//        }
//    }
//    setSelected(false);
//}
