//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef CHILDTABLEMODEL_H
#define CHILDTABLEMODEL_H

#include <QAbstractItemModel>
#include "gat-core_global.h"

class GATCORESHARED_EXPORT ChildTableModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    ChildTableModel(QObject *parent = nullptr);
    void setRootIndex(const QModelIndex &);
    void setColumn(const int column);
    void setRow(const int row);
    void setModel(QAbstractItemModel *);
    QModelIndex getRootIndex();

    int rowCount(const QModelIndex&parent = QModelIndex())const override;
    int columnCount(const QModelIndex& parent = QModelIndex())const override;
    QVariant data(const QModelIndex& index, const int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex & index) const override;
    QModelIndex index(int row, int column, const QModelIndex & parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex& index) const override;
    bool removeRows(int row, int count, const QModelIndex& parent = QModelIndex()) override;
    bool removeRow(int row, const QModelIndex &parent = QModelIndex());

public slots:
    void parentModelDataChanged(const QModelIndex&, const QModelIndex&);
    void forwardRowsAboutToBeRemoved(const QModelIndex &parent, int first, int last);
    void forwardRowsRemoved(const QModelIndex &parent, int first, int last);
    //void forwardRowsAboutToBeInserted(const QModelIndex&, int first, int last);
    void forwardRowsInserted(const QModelIndex&, int first, int last);

private:
    bool hasIndex(const int, const int) const;
    QAbstractItemModel* mParentModel;
    QModelIndex mRootIndex;
    int mModelColumn;
};

#endif // CHILDTABLEMODEL_H
