//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef CROSSSECTION_H
#define CROSSSECTION_H

#include <QString>
#include <vector>
#include <memory>
#include "projectedpoint.h"
#include <QVector>
#include "hydraulicparameters.h"
#include "profile.h"

class CrossSection : public Profile
{
public:
    CrossSection();
    CrossSection(const QString&);
    CrossSection(std::unique_ptr<Profile> profile);

    bool removeHydraulicParameters(const int row, const int count);

    void setD84(double d84);
    void setSlope(const double slope);
    void setMannings(const int i, double mannings);
    void setStageDescription(const int i, QString value);

    void setStage(const int, const double);

    int getHydraulicParameterCount() const;
    double getMaxStationing() const;
    double getd84() const;
    double getSlope() const;

    //Hydraulic Parameters
    double getStage(const int) const;
    QString getStageDescription(const int) const;
    double getMaxDepth(const int) const;
    double getArea(const int) const;
    double getWettedPerimeter(const int) const;
    double getWidth(const int) const;
    double getManningN(const int) const;   
    double getHydraulicRadius(const int i) const;
    double getMeanDepth(const int i) const;
    double getRelativeRoughness(const int i) const;
    double getMeanVelocity(const int i) const;
    double getDischarge(const int i) const;
    double getShearStress(const int i) const;
    double getStreamPower(const int i) const;
    double getFroudeNumber(const int i) const;
    double getFrictionFactor(const int i) const;
    double getShearVelocity(const int i) const;

    void setHydraulicParameterList(std::unique_ptr<std::vector<std::unique_ptr<HydraulicParameters>>>);

    void addHydraulicParameters(const double minStage, const double maxStage, const double increment);
    void addHydraulicParameters(const double stage);

private:
    //std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> mCrossSectionPointVector;

    std::unique_ptr<std::vector<std::unique_ptr<HydraulicParameters>>> mHydraulicParameters;
    double mSlope;
    double mD84;

    double calculateArea(const double stage);
    double calculateWettedPerimeter(const double stage);
    double calculateLength(const double w, const double h) const;
    double calculateSectionArea(const double x1, const double y1, const double x2, const double y2, const double stage) const;
    double calculateIntersect(const double x1, const double y1, const double x2, const double y2, const double stage) const;
    double calculateWidth(const double stage);
    double calculateSegmentWidth(const double stage, const int i);
    double calculateWetterPerimeterHeight(const double stage, const int i);
    void calculateHydraulicParameters(const int);

};

#endif // CROSSSECTION_H
