//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef CROSSSECTIONDAO_H
#define CROSSSECTIONDAO_H

#include <memory>
#include <vector>

#include "projectedpoint.h"
#include "crosssection.h"
#include "hydraulicparameters.h"

class QSqlDatabase;

class CrossSectionDao
{
public:
    CrossSectionDao(QSqlDatabase&);
    void init() const;
    int addCrossSection(std::unique_ptr<CrossSection> &) const;
    void removeCrossSection(const int id) const;
    void updateCrossSection(std::unique_ptr<CrossSection> &) const;
    void removeCrossSectionHydraulicParameters(const int id) const;
    bool removeMeasurements(const std::unique_ptr<CrossSection> &xs, int row, int count) const;
    std::unique_ptr<std::vector<std::unique_ptr<CrossSection>>> getCrossSections() const;
    void addCrossSectionHydraulicParameters(std::unique_ptr<CrossSection>& xs, int i) const;
    void addMeasurement(std::unique_ptr<CrossSection> &xs, int i) const;
    void updateCrossSectionPoints(std::unique_ptr<CrossSection>&) const;
    void updateProfilePoint(const std::unique_ptr<CrossSection> & xs, const int pn)const;

    void updateCrossSectionHydraulicParameters(std::unique_ptr<CrossSection> &xs) const;
    void updateMeasurement(std::unique_ptr<CrossSection>& xs, int i) const;

private:
    QSqlDatabase& mDatabase;
    std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> parseLineStringGeometry(QString &) const;
    std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> getPoints(const int xsId) const;
    QString toWKT(std::unique_ptr<CrossSection> &) const;
    std::unique_ptr<std::vector<std::unique_ptr<HydraulicParameters>>> getHydraulicParameters(const int crossSectionId) const;
    std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement>>> getProfileMeasurments(const int crossSectionId) const;

    void addCrossSectionPoints(std::unique_ptr<CrossSection> &) const;
    void removeCrossSectionPoints(const int id) const;

};

#endif // CROSSSECTIONDAO_H
