//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "gatdatamanager.h"
#include <QSqlDatabase>
#include <QDebug>
#include <QSqlQuery>
#include <QSqlError>



void GATDataManager::debugQuery(const QSqlQuery& query)
{
    if (query.lastError().type() == QSqlError::ErrorType::NoError) {
        qDebug() << "Query OK:"  << query.lastQuery();
    } else {
        qWarning() << "Query KO:" << query.lastError().text();
        qWarning() << "Query text:" << query.lastQuery();
    }
}


GATDataManager& GATDataManager::instance()
{

    static GATDataManager singleton;
    return singleton;
}

GATDataManager::GATDataManager(const QString & path):
    mDatabase(new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE"))),
    pointsDao(*mDatabase),
    alignmentDao(*mDatabase),
    crossSectionDao(*mDatabase),
    programDao(*mDatabase),
    longProfileDao(*mDatabase)
{
    if(path != ""){
        qDebug() << "GATDataManager::GATDataManger path: " << path;
        mDatabase->setDatabaseName(path);
        //TODO: re-execute this
        bool openStatus = mDatabase->open();
        qDebug() << "Database connection: " << (openStatus ? "OK" : "ERROR");
        //TODO: raise status exception

        pointsDao.init();
        alignmentDao.init();
        crossSectionDao.init();
        longProfileDao.init();
        programDao.init();
    }
}

void GATDataManager::setDatabaseFile(const QString & path){
    mDatabase->close();
    mDatabase->setDatabaseName(path);
    //TODO: re-execute this
    bool openStatus = mDatabase->open();
    qDebug() << "Database connection: " << (openStatus ? "OK" : "ERROR");
    //TODO: raise status exception

    pointsDao.init();
    alignmentDao.init();
    crossSectionDao.init();
    longProfileDao.init();
    programDao.init();

}

GATDataManager::~GATDataManager(){
    mDatabase->close();
}
