//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef GATDATAMANAGER_H
#define GATDATAMANAGER_H

#include <QString>
#include <memory>

#include "pointsdao.h"
#include "alignmentdao.h"
#include "crosssectiondao.h"
#include "longprofiledao.h"
#include "programdao.h"

class QSqlDatabase;
class QSqlQuery;


class GATDataManager
{

public:
    static void debugQuery(const QSqlQuery& query);
    static GATDataManager& instance();
    ~GATDataManager();
    void setDatabaseFile(const QString & path);

protected:
    GATDataManager(const QString & path = "");
    GATDataManager & operator=(const GATDataManager& rhs);

private:
    std::unique_ptr<QSqlDatabase> mDatabase;

public:
    const PointsDao pointsDao;
    const AlignmentDao alignmentDao;
    const CrossSectionDao crossSectionDao;
    const ProgramDao programDao;    
    const LongProfileDao longProfileDao;
};

#endif // GATDATAMANAGER_H
