//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "hydraulicparametersmodel.h"
#include "crosssection.h"
//needed for the enum.
#include "profilemodel.h"

HydraulicParametersModel::HydraulicParametersModel(QObject *parent) : ChildTableModel(parent)
{

}


QVariant HydraulicParametersModel::headerData(int section, const Qt::Orientation orientation, int role) const
{
    if(role == Qt::DisplayRole){
        if(orientation == Qt::Horizontal){
            switch(section){
            case ProfileModel::Description:
                return "Description";
            case ProfileModel::Stage:
                return "Stage";
            case ProfileModel::DepthMax:
                return "Max Depth";
            case ProfileModel::Area:
                return "Area";
            case ProfileModel::WettedPerimeter:
                return "Wetted Perimeter";
            case ProfileModel::Width:
                return "Width";
            case ProfileModel::HydraulicRadius:
                return "Hydraulic Radius";
            case ProfileModel::DepthMean:
                return "Mean Depth";
            case ProfileModel::ManningsN:
                return "Manning's N";
            case ProfileModel::RelativeRoughness:
                return "Relative Roughness";
            case ProfileModel::VelocityMean:
                return "Mean Velocity";
            case ProfileModel::FrictionFactor:
                return "Friction Factor";
            case ProfileModel::ShearVelocity:
                return "Shear Velocity";
            case ProfileModel::Discharge:
                return "Discharge";
            case ProfileModel::ShearStress:
                return "Shear Stress";
            case ProfileModel::StreamPower:
                return "Stream Power";
            case ProfileModel::FroudeNumber:
                return "Froude Number";

            default:
                return QVariant();
            }
        }
        if(orientation == Qt::Vertical){
            return section;
        }
    }
    if(role==Qt::StatusTipRole){
        if(orientation == Qt::Horizontal){
            switch(section){
            case ProfileModel::Description:
                return "Description - User Entered";
            case ProfileModel::Stage:
                return "Stage - User Entered";
            case ProfileModel::DepthMax:
                return "Max Depth - (Stage - minimum elevation)";
            case ProfileModel::Area:
                return "Area - sum of the section by section area below stage";
            case ProfileModel::WettedPerimeter:
                return "Wetted Perimeter - sum of the section by section wetted perimeter below stage";
            case ProfileModel::Width:
                return "Width - sum of section by section width below stage";
            case ProfileModel::HydraulicRadius:
                return "Hydraulic Radius - area / wetted pereimeter";
            case ProfileModel::DepthMean:
                return "Mean Depth - area/widh";
            case ProfileModel::ManningsN:
                return "Manning's N - user entered default 0.035";
            case ProfileModel::RelativeRoughness:
                return "Relative Roughness - hydraulic radius/D84";
            case ProfileModel::VelocityMean:
                return "Mean Velocity - from Manning's Equation";
            case ProfileModel::FrictionFactor:
                return "Friction Factor - mean velocity/shear velocity";
            case ProfileModel::ShearVelocity:
                return "Shear Velocity - (Shear Stress)^(1/2)";
            case ProfileModel::Discharge:
                return "Discharge - mean velocity * area";
            case ProfileModel::ShearStress:
                return "Shear Stress - 62.4*Hydraulic Radius*Slope";
            case ProfileModel::StreamPower:
                return "Stream Power - Shear Stress*Mean Velocity";
            case ProfileModel::FroudeNumber:
                return "Froude Number - mean velocity / ((32.2*mean depth)^(1/2))";

            default:
                return QVariant();
            }
        }
    }

    return QVariant();
}
