//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef LONGPROFILEDAO_H
#define LONGPROFILEDAO_H

#include <memory>
#include <vector>

#include "projectedpoint.h"
#include "crosssection.h"

class QSqlDatabase;

class LongProfileDao
{
public:
    LongProfileDao(QSqlDatabase&);
    void init() const;
    int addLongProfile(std::unique_ptr<CrossSection> &) const;
    void updateLongProfile(std::unique_ptr<CrossSection> &) const;
    void removeLongProfie(const int id) const;
    std::unique_ptr<std::vector<std::unique_ptr<CrossSection>>> getLongProfiles() const;
    void updateProfilePoints(std::unique_ptr<CrossSection> & profile) const;
    void updateProfilePoint(const std::unique_ptr<CrossSection> & profile, const int pn)const;

    //measurement functions
    bool removeMeasurements(const std::unique_ptr<CrossSection> &xs, int row, int count) const;
    void addMeasurement(std::unique_ptr<CrossSection> &xs, int i) const;
    void updateMeasurement(std::unique_ptr<CrossSection>& xs, int i) const;
private:
    QSqlDatabase& mDatabase;
    std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> getPoints(const int xsId) const;
    void addProfilePoints(std::unique_ptr<CrossSection> &) const;
    void removeProfilePoints(const int id) const;

    //measurement functions
    std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement>>> getProfileMeasurments(const int crossSectionId) const;
};

#endif // LONGPROFILEDAO_H
