//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "point.h"

Point::Point()
{
    northing = 0;
    easting = 0;
    elevation = 0;
    description = "Invalid";
    pointNumber=-9;
}

Point::Point(const double n,
             const double e,
             const double el,
             const QString &des)
{
    pointNumber = -9;
    northing = n;
    easting = e;
    elevation = el;
    description = des;
}

Point::Point(const double n, const double e, const double el)
{
    northing = n;
    easting = e;
    elevation = el;
    description = "Point by nez";
    pointNumber = -9;
}

Point::Point(const Point &copyPoint)
{
    northing = copyPoint.northing;
    easting = copyPoint.easting;
    elevation = copyPoint.elevation;
    description = copyPoint.description;
    pointNumber = copyPoint.pointNumber;
}

void Point::offsetElevation(const double offset)
{
    elevation = elevation + offset;
}

void Point::setDescription(const QString &des)
{
    description = des;
}

void Point::setPointNumber(const int pn)
{
    pointNumber = pn;
}

void Point::setNorthing(const double n)
{
    northing = n;
}

void Point::setEasting(const double e)
{
    easting = e;
}

void Point::setElevation(const double el)
{
    elevation = el;
}

double Point::getElevation() const
{
    return elevation;
}

double Point::getNorthing() const
{
    return northing;
}

double Point::getEasting() const
{
    return easting;
}

int Point::getPointNumber() const
{
    return pointNumber;
}

QString Point:: getDescription() const
{
    return description;
}

QPointF Point::getPointF()
{
    return QPointF(easting, northing);
}
