//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef POINTSDAO_H
#define POINTSDAO_H

#include <vector>
#include <memory>

#include "point.h"

class QSqlDatabase;
class QSqlQuery;

class PointsDao
{
public:

    PointsDao(QSqlDatabase& database);
    void init() const;
    void addPoint(Point& point) const;
    void addPoints(std::unique_ptr<std::vector<std::unique_ptr<Point>>> &pointList, int first, int last)const;
    void updatePoint(const std::unique_ptr<Point> & point) const;
    void removePoint(const int id) const;
    std::unique_ptr<std::vector<std::unique_ptr<Point>>> getPoints() const;
    std::unique_ptr<std::vector<std::unique_ptr<Point>>> getFilteredPoints(const double minX,
                                                                           const double minY,
                                                                           const double maxX,
                                                                           const double maxY,
                                                                       const QString &descriptionFilters) const;
private:
    QSqlDatabase& mDatabase;
    std::unique_ptr<std::vector<std::unique_ptr<Point>>> runQuery(QSqlQuery&) const;
};

#endif // POINTSDAO_H
