//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef POINTSMODEL_H
#define POINTSMODEL_H

#include <QAbstractListModel>
#include <QHash>
#include <vector>
#include <memory>
#include "point.h"
#include "gatdatamanager.h"
#include "gatcoordinatetransform.h"
#include "gat-core_global.h"

class GATCORESHARED_EXPORT PointsModel : public QAbstractListModel
{
   Q_OBJECT
public:

    enum Roles{
        IdRole = Qt::UserRole + 1,
        NorthingRole,
        EastingRole,
        ElevationRole,
        DescriptionRole,
    };

    PointsModel(QObject* parent = nullptr);

    QModelIndex addPoint(const Point& point);

    Q_INVOKABLE bool addPointsFromFile(const QString & delimter, const QString& url);
    int rowCount(const QModelIndex& parent = QModelIndex()) const override;
    int columnCount(const QModelIndex& parent = QModelIndex())const override;
    QVariant data(const QModelIndex& index, const int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
    bool removeRows(int row, int count, const QModelIndex& parent) override;
    QHash<int, QByteArray> roleNames() const override;
    Qt::ItemFlags flags(const QModelIndex & index) const override;

public slots:
    Q_INVOKABLE void addPointFromNEZD(const double n, const double e, const double el, const QString d);
    Q_INVOKABLE void addPointFromMouse(const QPointF);
    void databaseUpdated();

private:
    bool isIndexValid(const QModelIndex& index) const;

private:
    GATDataManager & mDb;
    GATCoordinateTransform & mCoordinateTransform;
    std::unique_ptr<std::vector<std::unique_ptr<Point>>> mPoints;
};

#endif // POINTSMODEL_H
