#ifndef PROFILE_H
#define PROFILE_H

#include <memory>
#include <vector>

#include "projectedpoint.h"
#include "profilemeasurement.h"

//TODO:
//Create a Longitudinal Profile
//Adjust Cross Section to inherit from this class.

class Profile
{
public:
    Profile();
    Profile(const QString &name);
    Profile(std::unique_ptr<Profile> profile);

    //profile parameters
    void setName(const QString &);
    void setTolerance(const double tolerance);
    void setFilterString(const QString filterString);
    void setId(const int id);
    void setAlignmentId(const int id);
    void setPlot(const bool);
    void setChannelFeature(const int feat);

    QString getName() const;
    double getTolerance() const;
    QString getFilterString() const;
    int getId() const;
    int getAlignmentId() const;
    bool getPlot() const;
    int getChannelFeature() const;

    //point paramters
    bool removePoint(const int pn);
    int getPointCount() const;
    void addPoint(std::unique_ptr<ProjectedPoint> point);
    void setPointVector(std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>>);
    QRectF getBoundingRect();

    void sortPointVector();

    //measurement parameters
    void setMeasurementList(std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement> > >);
    int getMeasurementCount() const;

    qreal getPointOneX(const int pn) const;
    qreal getPointOneY(const int pn) const;
    qreal getPointTwoX(const int pn) const;
    qreal getPointTwoY(const int pn) const;
    int getPointType(const int pn) const;
    qreal getMeasurementValue(const int pn) const;
    int getMeasurementId(const int pn) const;

    void setPointOneX(qreal value, const int pn);
    void setPointTwoX(qreal value, const int pn);
    void setPointOneY(qreal value, const int pn);
    void setPointTwoY(qreal value, const int pn);
    void setPointType(int type, const int pn);


    bool removeMeasurements(const int row, const int count);
    void setMeasurementId(const int pn, const int id) const;

    void addMeasurementPoint(const QPointF &pointOne, const QPointF &pointTwo);

    //individual point information
    void setNorthing(const int, const double);
    void setEasting(const int, const double);
    void setElevation(const int, const double);
    void setStationing(const int, const double);

    double getNorthing(const int) const;
    double getEasting(const int) const;
    double getElevation(const int) const;
    double getStationing(const int) const;
    int getOriginatingPointId(const int) const;
    QString getDescription(const int) const;

    void calculateStationing(const int startPoint);
    bool removePoints(const int pn, const int count);

    double getMaxStationing() const;
    qreal getMinPointElevation() const;
    qreal getMaxPointElevation() const;

    void calculateMinPointElevation();
    void calculateMaxPointElevation();
private:
    std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> mProfilePointVector;
    std::unique_ptr<std::vector<std::unique_ptr<ProfileMeasurement>>> mProfileMeasurements;
    int mId;
    QString mName;
    int mAlignmentId;
    double mTolerance;
    QString mFilterString;
    bool mPlot;
    qreal mMinPointElevation;
    qreal mMaxPointElevation;
    int mChannelFeature;

    bool isPointValid(const int pn) const;
    bool isMeasurementPointValid(const int pn)const;

    QPointF getPointF(const int pn) const;

};

#endif // PROFILE_H
