#include "profilecalculator.h"
#include "profileutilities.h"

ProfileCalculator::ProfileCalculator(): mDb(GATDataManager::instance())
{

}

std::unique_ptr<CrossSection> ProfileCalculator::createProfilePointsAlignment(const QModelIndex &index, AlignmentModel *AlignmentModel, const QString pointFilters, const double tolerance)
{
    qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment begin";
    std::unique_ptr <CrossSection> newProfile(new CrossSection);
    newProfile->setTolerance(tolerance);
    newProfile->setFilterString(pointFilters);
    qDebug() << "ProfileCalculate::createProfilePointsAlignment pointFilters input " << pointFilters;
    qDebug() << "set filter string " << newProfile->getFilterString();
    if(!AlignmentModel->isIndexValid(index)){
        return nullptr;
    }
    //pull alignment information
    //Alignment* alignment = mAlignmentModel->getAlignment(index);
    QModelIndex alignmentIdIndex = AlignmentModel->index(index.row(), 0, QModelIndex());
    newProfile->setAlignmentId(AlignmentModel->data(alignmentIdIndex).toInt());
    newProfile->setName(AlignmentModel->data(AlignmentModel->index(index.row(),1)).toString());
    //alignment->calculateStationing();
    //qDebug() << "Alignment stationing " << alignment->getStationing(0);
    //std::shared_ptr<std::vector<std::unique_ptr<Point2D>>> alignmentPoints = alignment.getPointVector();
    if(AlignmentModel->rowCount(index) == 0){
        qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment returning nullptr";
        return nullptr;
    }
    //calculate bounding box
    double minX;
    double maxX;
    double minY;
    double maxY;

    //normalize bounding rectangle
    QRectF boundingRect = AlignmentModel->data(AlignmentModel->index(index.row(),2)).toRectF();
    if(boundingRect.left()<boundingRect.right()){
        minX = boundingRect.left() - tolerance;
        maxX = boundingRect.right() + tolerance;
    }
    else{
        minX = boundingRect.right() - tolerance;
        maxX = boundingRect.left() - tolerance;
    }

    if(boundingRect.top() > boundingRect.bottom()){
        minY = boundingRect.bottom()- tolerance;
        maxY = boundingRect.top() + tolerance;
    }
    else {
        minY = boundingRect.top() - tolerance;
        maxY = boundingRect.bottom() + tolerance;
    }

    double tolSq = tolerance * tolerance;
    //qDebug() << "CrossSectionModel::createCrossSectionPointsAlignmnet calculated the tolerance " << tolSq;
    //Select applicable points based on description and tolerance
    std::unique_ptr<std::vector<std::unique_ptr<Point>>> points = mDb.pointsDao.getFilteredPoints(minX, minY, maxX, maxY, pointFilters);

    if(points == nullptr){
        //If there are no points there is nothing to do.
        return nullptr;
    }
    if(points->size() == 0){
        //If there are not points a cross section won't be generated.
        return nullptr;
    }
    //qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment have filtered points " << points->size();
    for(int i = 0; i< static_cast<int>(points->size()); i++){
        //qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment begin itteration of points";
        //all the points should be closer than this due to the bounding box
        double distanceSq = tolSq+.01;
        int closestSegment = -9;
        QPointF closestIntersectPoint;
        QPointF pointOfInterest(points->at(static_cast<size_t>(i))->getEasting(), points->at(static_cast<size_t>(i))->getNorthing());
        for(int j = 0; j< AlignmentModel->rowCount(index)-1; j++){
            //qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment begin itteration of alignment segments";
            QModelIndex indexNorthing0 = AlignmentModel->index(j,0,index);
            QModelIndex indexEasting0 = AlignmentModel->index(j,1,index);
            QModelIndex indexPointType = AlignmentModel->index(j, 3, index);
            QModelIndex indexNorthing1 = AlignmentModel->index(j+1,0,index);
            QModelIndex indexEasting1 = AlignmentModel->index(j+1,1,index);

            QPointF point0 = QPointF(AlignmentModel->data(indexEasting0).toDouble(), AlignmentModel->data(indexNorthing0).toDouble());
            QPointF point1 = QPointF(AlignmentModel->data(indexEasting1).toDouble(), AlignmentModel->data(indexNorthing1).toDouble());
            AlignmentPoint::PointType pointType = static_cast<AlignmentPoint::PointType>(AlignmentModel->data(indexPointType).toInt());
            QLineF lineSegment(point0, point1);
            QPointF intersectPoint;
            if(pointType == AlignmentPoint::Line){
                intersectPoint = GAT::calculateIntersectPointLine(lineSegment, pointOfInterest);
            }
            else{
                QModelIndex indexCenterNorthing0 = AlignmentModel->index(j,4,index);
                QModelIndex indexCenterEasting0 = AlignmentModel->index(j,5,index);
                QPointF center = QPointF(AlignmentModel->data(indexCenterEasting0).toDouble(), AlignmentModel->data(indexCenterNorthing0).toDouble());
                intersectPoint = GAT::calculateIntersectPointCurve(lineSegment, center, pointOfInterest);
            }

            double segmentDistanceSq = GAT::calculateDistanceSq(pointOfInterest, intersectPoint);

            if(segmentDistanceSq < distanceSq){
                distanceSq = segmentDistanceSq;
                closestSegment = j;
                closestIntersectPoint = intersectPoint;
            }
        }
        qDebug() << "CrossSectionModel::createCrossSectionPointsAlignment getting ready to add point";
        if(closestSegment != -9 && distanceSq <= tolSq){
            //Calculate intersect point
            std::unique_ptr <ProjectedPoint> newPoint(new ProjectedPoint);
            newPoint->setOriginatingPointId(points->at(static_cast<unsigned int>(i))->getPointNumber());
            newPoint->setClosestSegment(closestSegment);
            newPoint->setEasting(closestIntersectPoint.x());
            newPoint->setNorthing(closestIntersectPoint.y());
            newPoint->setElevation(points->at(static_cast<unsigned int>(i))->getElevation());
            newPoint->setOriginatingPointId(points->at(static_cast<unsigned int>(i))->getPointNumber());
            newPoint->setDescription(points->at(static_cast<unsigned int>(i))->getDescription());
            qDebug() << "Description: " << newPoint->getDescription();
            //Append this to a list of points
            //This will make the segment become just the piece from the alignment point to the intersect point
            QLineF lineSegment;
            lineSegment.setP2(closestIntersectPoint);
            QModelIndex indexNorthing0 = AlignmentModel->index(closestSegment,0,index);
            QModelIndex indexEasting0 = AlignmentModel->index(closestSegment,1,index);
            lineSegment.setP1(QPointF(AlignmentModel->data(indexEasting0).toDouble(), AlignmentModel->data(indexNorthing0).toDouble()));

            QModelIndex indexStationing = AlignmentModel->index(closestSegment, 2, index);
            newPoint->setStation(AlignmentModel->data(indexStationing).toDouble()+lineSegment.length());

            newProfile->addPoint(move(newPoint));
        }
    }
    //Calculated distance from each segment
    newProfile->sortPointVector();
    qDebug() << "ProfileCalculator::createProfilePointsAlignment point filters end : " << newProfile->getFilterString();
    return newProfile;
}
