#include "profiledao.h"
#include <QSqlQuery>
#include <QDebug>

#include "gatdatamanager.h"

ProfileDao::ProfileDao(QSqlDatabase & database): mDatabase(database)
{

}

void ProfileDao::init() const
{
    if(!mDatabase.tables().contains("profiles")){
        QSqlQuery query(mDatabase);
        query.exec("CREATE TABLE profiles (profileID INTEGER PRIMARY KEY AUTOINCREMENT,"
                   "description TEXT,"
                   "tolerance REAL,"
                   "filterString TEXT,"
                   "lastModified DATETIME)");
        GATDataManager::debugQuery(query);
    }
    if(!mDatabase.tables().contains("profilePoints")){
        QSqlQuery query(mDatabase);
        query.exec("CREATE TABLE profilePoints("
                   "northing REAL, "
                   "easting REAL, "
                   "elevation REAL, "
                   "sortOrder INTEGER NOT NULL, "
                   "description TEXT, "
                   "pointNumber REAL NULL REFERENCES profiles (profileId), "
                   "lastModified DATETIME)");
        GATDataManager::debugQuery(query);
    }
}

std::unique_ptr<std::vector<std::unique_ptr<Profile> > > ProfileDao::getProfiles()
{
    QSqlQuery query("SELECT * FROM profiles", mDatabase);
    query.exec();
    std::unique_ptr<std::vector<std::unique_ptr<Profile>>> list(new std::vector<std::unique_ptr<Profile>>());
    while(query.next()){
        std::unique_ptr<Profile> xs(new Profile());
        xs->setId(query.value("profileID").toInt());
        xs->setName(query.value("description").toString());
        xs->setTolerance(query.value("tolerance").toDouble());
        xs->setFilterString(query.value("filterString").toString());
        //QString lineString = query.value("WKT").toString();
        xs->setPointVector(getPoints(xs->getId()));
        list->push_back(move(xs));
    }

    return list;
}

std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint> > > ProfileDao::getPoints(const int xsId) const
{
    std::unique_ptr<std::vector<std::unique_ptr<ProjectedPoint>>> list(new std::vector<std::unique_ptr<ProjectedPoint>>());
    QSqlQuery query("SELECT northing, easting, elevation, stationing, description, pointNumber FROM crossSectionPoints WHERE crossSectionID = ? ORDER BY sortOrder", mDatabase);
    query.bindValue(0, xsId);
    query.exec();
    GATDataManager::debugQuery(query);
    while(query.next()){
        std::unique_ptr<ProjectedPoint> newPoint(new ProjectedPoint);
        newPoint->setNorthing(query.value("northing").toDouble());
        newPoint->setEasting(query.value("easting").toDouble());
        newPoint->setElevation(query.value("elevation").toDouble());
        newPoint->setStation(query.value("stationing").toDouble());
        newPoint->setDescription(query.value("description").toString());
        newPoint->setOriginatingPointId(query.value("pointNumber").toInt());
        list->push_back(move(newPoint));
    }
    return list;
}
