#include "profileutilities.h"


double GAT::calculateDistanceSq(const QPointF pointOne, const QPointF pointTwo)
{
    return (pointOne.x() - pointTwo.x())*(pointOne.x() - pointTwo.x())+(pointOne.y() - pointTwo.y())*(pointOne.y() - pointTwo.y());
}

QPointF GAT::calculateIntersectPointLine(QLineF &line, QPointF &point)
{
    //July 22, 2019 Updated equation for projecting the point to correct a math error.
    //qDebug() << "CrossSectionModel::calculateIntersectPoint line then point " << line << " " << point;
    QPointF newPoint;
    //check for zero slope if it is a vertical line then the interset point
    //is the x point on the line (they are the same) and the y point is the same as the point.

    if(line.dx() == 0){
        qDebug() << "Line Segment is vertical";
        newPoint.setX(line.x1());
        if(point.y() < line.y1() && point.y() < line.y2()){
            if(line.y1() < line.y2()){
                newPoint.setY(line.y1());
            }
            else{
                newPoint.setY(line.y2());
            }
        }
        else if(point.y() > line.y1() && point.y() > line.y2()){
            if(line.y1() > line.y2()){
                newPoint.setY(line.y1());
            }
            else{
                newPoint.setY(line.y2());
            }
        }
        else {
            newPoint.setY(point.y());
        }
    }
    else if(line.dy() == 0){
        qDebug() << "Line Segment is horizontal";
        newPoint.setY(line.y1());
        if(point.x() < line.x1() && point.x() < line.x2()){
            if(line.x1() < line.x2()){
                newPoint.setX(line.x1());
            }
            else{
                newPoint.setX(line.x2());
            }
        }
        else if(point.x() > line.x1() && point.x() > line.x2()){
            if(line.x1() > line.x2()){
                newPoint.setX(line.x1());
            }
            else{
                newPoint.setX(line.x2());
            }
        }
        else {
            newPoint.setX(point.x());
        }
    }
    //If it isn't a vertical line then the slope is a number find the intersect point
    else
    {
        double A = line.y2() - line.y1();
        double B = line.x2() - line.x1();
        double C = point.y() - line.y1();
        qDebug() << "A B C" << A << " " << B << " " << C;
        qDebug() << "Calculating D and E";
        //Simplify to reduce errors
        double D= (A*A+B*B);
        double E = A*B*C+A*A*line.x1()+B*B*point.x();
        newPoint.setX(E/D);
        newPoint.setY(A/B*(newPoint.x()-line.x1())+line.y1());
        qDebug() << "X" << newPoint.x();
        //Check to see if the intersection point is within the segment, if not the closest point
        //occurs on an endpoint.
        if((newPoint.x() < line.x1() && newPoint.x() < line.x2())){
            qDebug() << "Returning endpoint due to x being less than both x points";
            if(line.x1() < line.x2()){
                //return the one with the smaller x as the closest point
                return line.p1();
            }
            else
            {
                return line.p2();
            }
        }
        else if((newPoint.x() > line.x1() && newPoint.x() > line.x2())){
            qDebug() << "Returning endpoint due to x being greater than both x points";
            if(line.x1() > line.x2()){
                return line.p1();
            }
            else{
                return line.p2();
            }
        }
    }
    qDebug() << "Insersect point " << newPoint;
    return newPoint;
}

QPointF GAT::calculateIntersectPointCurve(QLineF &chord, QPointF &center, QPointF &point)
{
    qreal thetaPoint = qAtan2(point.y()-center.y(), point.x()-center.x());
    qreal thetaBegin = qAtan2(chord.p1().y()-center.y(), chord.p1().x()-center.x());
    qreal thetaEnd = qAtan2(chord.p2().y()-center.y(), chord.p2().x()-center.x());
    bool inbetween = false;
    if(thetaBegin < thetaEnd ){
        if(thetaPoint <= thetaEnd && thetaPoint >= thetaBegin){
            inbetween = true;
        }
    }
    else {
        if(thetaPoint > thetaBegin){
            inbetween = true;
        }
        if(thetaPoint < thetaEnd){
            inbetween = true;
        }
    }
    if(inbetween){
        qreal slope = (point.y()-center.y())/(point.x()-center.x());
        qreal slopeSq = slope*slope;
        qreal radius = QLineF(center, chord.p1()).length();
        qreal radiusSq = radius*radius;
        qreal x = qSqrt(radiusSq/(slopeSq+1));
        qreal y = qSqrt(radiusSq*slopeSq/(slopeSq+1));
        if(thetaPoint > M_PI_2 && thetaPoint < 3*M_PI_2){
            //if the angle is greater than pi/2 and less than 3pi/2 then x is negative
            x=-1*x;
        }
        if(thetaPoint > M_PI){
            y=-1*y;
        }
        //shift based on center
        x = x + center.x();
        y = y + center.y();
        return QPointF(x,y);
    }
    if(QLineF(chord.p1(),point).length()<QLineF(chord.p2(),point).length()){
        return chord.p1();
    }
    //default
    return chord.p2();
}

qreal GAT::spanAngle(const QPointF &pointOne, const QPointF &pointTwo, const QPointF &center){

    qreal thetaBegin = qAtan2(pointOne.y()-center.y(), pointOne.x()-center.x());
    qreal thetaEnd = qAtan2(pointTwo.y()-center.y(), pointTwo.x()-center.x());

    //if thetaBegin is greater then it started before 2Pi
    if(thetaBegin > thetaEnd){
        return 2*M_PI-thetaBegin+thetaEnd;
    }

    return thetaEnd-thetaBegin;
}

