//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "projectedpoint.h"
#include <QDebug>

ProjectedPoint::ProjectedPoint()
{

}

ProjectedPoint::ProjectedPoint(const ProjectedPoint & xs)
{
    mNorthing = xs.getNorthing();
    mEasting = xs.getEasting();
    mElevation = xs.getElevation();
    mStation = xs.getStationing();
    mClosestSegment = xs.getClosestSegment();
    mOrigintatingPointId = xs.getOriginatingPointId();
    //qDebug() << "CrossSectionPoint::CrossSectionPoint description " << xs.getDescription();
    mDescription = xs.getDescription();
}

void ProjectedPoint::setNorthing(const double n)
{
    mNorthing = n;
}

void ProjectedPoint::setEasting(const double e)
{
    mEasting = e;
}

void ProjectedPoint::setElevation(const double el)
{
    mElevation = el;
}

void ProjectedPoint::setStation(const double st)
{
    mStation = st;
}

void ProjectedPoint::setDescription(const QString &st)
{
    mDescription = st;
}

double ProjectedPoint::getNorthing() const
{
    return mNorthing;
}

double ProjectedPoint::getEasting() const
{
    return mEasting;
}

double ProjectedPoint::getElevation() const
{
    return mElevation;
}

double ProjectedPoint::getStationing()const
{
    return mStation;
}

QPointF ProjectedPoint::getPointF() const
{
    return(QPointF(mEasting, mNorthing));
}

void ProjectedPoint::setClosestSegment(const int id)
{
    mClosestSegment = id;
}

int ProjectedPoint::getClosestSegment() const
{
    return mClosestSegment;
}

void ProjectedPoint::setOriginatingPointId(const int id)
{
    mOrigintatingPointId = id;
}

int ProjectedPoint::getOriginatingPointId() const
{
    return mOrigintatingPointId;
}

QString ProjectedPoint::getDescription() const
{
    return mDescription;
}

void ProjectedPoint::copyPoint(const ProjectedPoint &copyPt)
{
    this->mEasting = copyPt.getEasting();
    this->mNorthing = copyPt.getNorthing();
    this->mElevation = copyPt.getElevation();
    this->mStation = copyPt.getStationing();
    this->mDescription = copyPt.getDescription();
}

//Sorts by stationing
bool ProjectedPoint::operator<(const ProjectedPoint &rhs) const
{
    return mStation < rhs.mStation;
}
