//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef PROJECTEDPOINT_H
#define PROJECTEDPOINT_H
#include <QString>
#include <QPointF>

class ProjectedPoint
{
public:
    //A cross-section point may have a northing, easting, elevation, and station.
    //Cross-sections are defined by the user projecting points onto an Alignment, or alternatively manually entering
    //the points.
    //The station forms a linear coordinate system
    ProjectedPoint();
    ProjectedPoint(const ProjectedPoint &);
    void setNorthing(const double);
    void setEasting(const double);
    void setElevation(const double);
    void setStation(const double);
    void setPointNumber(const int);
    void setDescription(const QString &);
    double getNorthing() const;
    double getEasting() const;
    double getElevation() const;
    double getStationing() const;
    double getPointNumber() const;
    QString getDescription() const;
    QPointF getPointF() const;
    void setClosestSegment(const int id);
    int getClosestSegment() const;
    void setOriginatingPointId(const int);
    int getOriginatingPointId() const;
    void copyPoint(const ProjectedPoint &);

    bool operator<(const ProjectedPoint &rhs)const;

private:
   double mNorthing;
   double mEasting;
   double mElevation;
   double mStation;
   int mClosestSegment;
   int mOrigintatingPointId;
   QString mDescription;
};

#endif // CROSSSECTIONPOINT_H
