#include "alignmentgraphicsitem.h"
#include <QPainter>
#include "alignmentlayer.h"
#include "gatplanviewscene.h"

AlignmentGraphicsItem::AlignmentGraphicsItem(QGraphicsItem* parent) : QGraphicsItem(parent),
    mCoordinateTransform(GATCoordinateTransform::instance())

{
    setAcceptTouchEvents(true);
    setAcceptedMouseButtons(Qt::LeftButton);
    mRadius = 5;
}

void AlignmentGraphicsItem::setPosition(double easting, double northing)
{
    mEasting = easting;
    mNorthing = northing;
    this->setPos();
}

void AlignmentGraphicsItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(widget);
    Q_UNUSED(option);
    double hScale = painter->transform().m11();
    mRadius = 5/hScale;
    QBrush brush(Qt::green);
    QPen pen(Qt::yellow,1/hScale);
    painter->setBrush(brush);
    painter->setPen(pen);
    painter->drawEllipse(this->boundingRect());
}

QRectF AlignmentGraphicsItem::boundingRect() const
{
    return QRectF(-mRadius, -mRadius, width(), height());
}

qreal AlignmentGraphicsItem::width() const
{
    return 2*mRadius;
}

qreal AlignmentGraphicsItem::height() const
{
    return 2*mRadius;
}

void AlignmentGraphicsItem::setPos()
{
    QPointF point = mCoordinateTransform.toPlottingCoordinates(QPointF(mEasting, mNorthing));
    QGraphicsItem::setPos(point);
}

void AlignmentGraphicsItem::setAlignmentLayer(AlignmentLayer *layer)
{
    mLayer = layer;
}

void AlignmentGraphicsItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    Q_UNUSED(event);
    switch (mLayer->getCurrentMode()){
        case GATPlanViewScene::deleteAlignmentPointMode:
        mLayer->deletePoint(this);
        break;
    case GATPlanViewScene::moveAlignmentPointMode:

        mMoving = true;
        break;

    default:
        break;
    }
}

void AlignmentGraphicsItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    if(mMoving){
        QGraphicsItem::setPos(event->scenePos());
    }
}

void AlignmentGraphicsItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    if(mMoving){
        QPointF point = mCoordinateTransform.toDataCoordinates(event->scenePos());
        mEasting = point.x();
        mNorthing = point.y();
        setPos();
        mLayer->updatePoint(this, mNorthing, mEasting);
        //TODO: Tell the group to emit signals
    }
    mMoving = false;

}
