
#ifndef ALIGNMENTLAYER_H
#define ALIGNMENTLAYER_H

#include <QGraphicsItemGroup>
#include <QObject>
#include "alignmentmodel.h"
#include <QPersistentModelIndex>
#include <QHash>
#include <alignmentlinegraphicsitem.h>
#include <alignmentgraphicsitem.h>
#include "gatplanviewscene.h"


class AlignmentLayer : public QObject, public QGraphicsItemGroup
{
    Q_OBJECT
public:
    AlignmentLayer(QGraphicsItem*parent = nullptr);
    void setModel(AlignmentModel* model);
    void loadAlignments();
    void updatePoint(AlignmentGraphicsItem *, qreal northing, qreal easting);
    void deletePoint(AlignmentGraphicsItem *);
    void insertPoint(AlignmentLineGraphicsItem *, double northing, double easting);
    GATPlanViewScene::GATSceneMode getCurrentMode();
    void setCurrentMode(GATPlanViewScene::GATSceneMode mode);
    QRectF boundingRect() const override;

public slots:
    void modelReset();
    void modelAboutToBeReset();
    void dataChanged(const QModelIndex&topLeft, const QModelIndex &bottomRight, const QVector<int> &roles);
    void rowsInserted(const QModelIndex &parent, int first, int last);
    void rowsAboutToBeRemoved(const QModelIndex&parent, int first, int last); 

private:
    AlignmentModel* mAlignmentModel;
    GATPlanViewScene::GATSceneMode mMode;
    QHash<AlignmentGraphicsItem*, QPersistentModelIndex> mIndexPointHash;
    QHash<AlignmentLineGraphicsItem*, QPersistentModelIndex> mIndexLineHash;
    void removePoint(AlignmentGraphicsItem* point);
    void removeLine(AlignmentLineGraphicsItem* line);
    void loadAlignmentPoints();
    void loadAlignmentLines();
    void clearAlignments();
    QRectF mBoundingRect;
    void calculateBoundingRect();
};

#endif // ALIGNMENTLAYER_H
