#include "alignmentlinegraphicsitem.h"
#include "gatplanviewscene.h"
#include "alignmentlayer.h"
#include <QDebug>
#include <QtMath>

AlignmentLineGraphicsItem::AlignmentLineGraphicsItem(QGraphicsItem *parent) :
    QGraphicsItem(parent), mCoordinateTransform(GATCoordinateTransform::instance())
{
    setPen(QPen(Qt::black, 2));
    setAcceptedMouseButtons(Qt::LeftButton);
}

void AlignmentLineGraphicsItem::setAlignmentLayer(AlignmentLayer *layer)
{
    mAlignmentLayer = layer;
}

void AlignmentLineGraphicsItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    //qDebug() << "AlignmentLineGraphicsItem::mousePressEvent";
   //qDebug() << "current mode " << mAlignmentLayer->getCurrentMode();
    if(mAlignmentLayer->getCurrentMode() == GATPlanViewScene::insertAlignmentPointMode){
        //qDebug() << "inserting point";
        QPointF point = mCoordinateTransform.toDataCoordinates(event->pos());
        mAlignmentLayer->insertPoint(this, point.y(), point.x());
    }
}

void AlignmentLineGraphicsItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    Q_UNUSED(event)
}

void AlignmentLineGraphicsItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(widget)
    Q_UNUSED(option)

    double hScale = painter->transform().m11();
    QPen scalePen = mPen;
    scalePen.setWidthF(mPen.widthF()/hScale);

    painter->setPen(scalePen);
    if(mPointType == AlignmentPoint::CurveLeft || mPointType == AlignmentPoint::CurveRight){
        qreal radius = sqrt((mBeginPoint.x()-mCenterPoint.x())*(mBeginPoint.x()-mCenterPoint.x())+(mBeginPoint.y()-mCenterPoint.y())*(mBeginPoint.y()-mCenterPoint.y()));
        //Angles are is 1/16 of degree (180*16 = 2880)

        int startAngle = static_cast<int>(qAtan2(mBeginPoint.y()-mCenterPoint.y(),mBeginPoint.x()-mCenterPoint.x())*2880/M_PI);
        int endAngle = static_cast<int>(qAtan2(mEndPoint.y()-mCenterPoint.y(),mEndPoint.x()-mCenterPoint.x())*2880/M_PI);
        //qDebug() << "AlignmentLineGraphicsItem::paint center point " << mCenterPoint;
        //qDebug() << "beginPoint " << mBeginPoint;
        //qDebug() << "endPoint " << mEndPoint;
       // qDebug() << "radius" << radius;
       // qDebug() << "startAngle " << startAngle;
        //qDebug() << "endAngle " << endAngle;
        QRectF rect(mCenterPoint.x()-radius, mCenterPoint.y()-radius, radius*2, radius*2);
        //qDebug() << "rect " << rect;
        //rect, start, span
        //Axis flipped so clockwise becomes positive
        int span = startAngle-endAngle;
        if(span>2880){
            span = span - 5760;
        }
        else if (span < -2880){
            span = span + 5760;
        }
        //drawArc is in degrees and with y axis positive down.
        painter->drawArc(rect,-1*startAngle, span);
    }
    if(mPointType == AlignmentPoint::Line){
        painter->drawLine(mBeginPoint, mEndPoint);
    }
}

void AlignmentLineGraphicsItem::setPen(const QPen &pen)
{
    mPen = pen;
}

void AlignmentLineGraphicsItem::setPointType(AlignmentPoint::PointType pointType)
{
    mPointType = pointType;
}

void AlignmentLineGraphicsItem::setLine(qreal easting0, qreal northing0, qreal easting1, qreal northing1)
{
    mBeginPoint.setX(easting0);
    mBeginPoint.setY(northing0);
    mEndPoint.setX(easting1);
    mEndPoint.setY(northing1);
}

void AlignmentLineGraphicsItem::setCenter(qreal easting, qreal northing)
{
    mCenterPoint.setX(easting);
    mCenterPoint.setY(northing);
}

QRectF AlignmentLineGraphicsItem::boundingRect() const
{
    QRectF boundRect;
    if(mPointType == AlignmentPoint::Line){
        if(mBeginPoint.x() < mEndPoint.x()){
            boundRect.setX(mBeginPoint.x());
            boundRect.setWidth(mEndPoint.x()-mBeginPoint.x());
        }
        else {
            boundRect.setX(mEndPoint.x());
            boundRect.setWidth(mBeginPoint.x()-mEndPoint.x());
        }

        if(mBeginPoint.y() < mEndPoint.y()){
            boundRect.setY(mBeginPoint.y());
            boundRect.setHeight(mEndPoint.y()-mBeginPoint.y());
        }
        else {
            boundRect.setY(mEndPoint.y());
            boundRect.setHeight(mBeginPoint.y()-mEndPoint.y());
        }
        return boundRect;
    }
    if(mPointType == AlignmentPoint::CurveLeft || mPointType == AlignmentPoint::CurveRight){
        qreal startAngle;
        qreal endAngle;
        if(mPointType == AlignmentPoint::CurveLeft){
            startAngle = static_cast<int>(qAtan2(mBeginPoint.y()-mCenterPoint.y(),mBeginPoint.x()-mCenterPoint.x()));
            endAngle = static_cast<int>(qAtan2(mEndPoint.y()-mCenterPoint.y(),mEndPoint.x()-mCenterPoint.x()));
        }
        else { //curve right
            endAngle = static_cast<int>(qAtan2(mBeginPoint.y()-mCenterPoint.y(),mBeginPoint.x()-mCenterPoint.x()));
            startAngle = static_cast<int>(qAtan2(mEndPoint.y()-mCenterPoint.y(),mEndPoint.x()-mCenterPoint.x()));
        }
        qreal minEasting;
        qreal maxEasting;
        qreal minNorthing;
        qreal maxNorthing;
        qreal radius = sqrt((mBeginPoint.x()-mCenterPoint.x())*(mBeginPoint.x()-mCenterPoint.x())+(mBeginPoint.y()-mCenterPoint.y())*(mBeginPoint.y()-mCenterPoint.y()));
        if(mBeginPoint.x() > mEndPoint.x()){
            maxEasting = mBeginPoint.x();
            minEasting = mEndPoint.x();
        }
        else{
            maxEasting = mEndPoint.x();
            minEasting = mBeginPoint.x();
        }

        if(mBeginPoint.y() > mEndPoint.y()){
            maxNorthing = mBeginPoint.y();
            minNorthing = mEndPoint.y();
        }
        else{
            maxNorthing = mEndPoint.y();
            minNorthing = mBeginPoint.y();
        }
        //qAtan2 - is in radians

        if(startAngle > endAngle){
            //Check 0 radians
            //normalize so startAngle is less than end angle
            if(radius > maxEasting){
                maxEasting = radius;
            }
            startAngle = startAngle - (2*M_PI);
        }
        if(startAngle < M_PI_2 && endAngle > M_PI_2){
            //check PI/2
            if(radius > maxNorthing){
                maxNorthing = radius;
            }
        }
        if(startAngle < M_PI && endAngle > M_PI){
            //Check PI
            if(-1*radius < minEasting){
                minEasting = -1*radius;
            }
        }
        if(startAngle < 3*M_PI_2 && endAngle > 3*M_PI_2){
            //Check 3*PI/2
            if(-1*radius < minNorthing){
                minNorthing = -1*radius;
            }
        }

        return QRectF(minEasting, minNorthing, maxEasting-minEasting, maxNorthing-minNorthing);
    }

    return QRectF();

}

