//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "checkboxitemdelegate.h"
#include <QCheckBox>
#include <QDebug>
#include <QPainter>
#include <QEvent>

CheckBoxItemDelegate::CheckBoxItemDelegate(QObject *parent)
    :QItemDelegate (parent)
{

}

QWidget *CheckBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(option)
    Q_UNUSED(index)
    QCheckBox* checkBox = new QCheckBox(parent);
    return checkBox;
}

void CheckBoxItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    Q_UNUSED(editor)
    Q_UNUSED(index)
//    QString value = index.model()->data(index, Qt::EditRole).toString();
//    qDebug() << "CheckBoxItemDelegate::setEditorData Value: " << value;
//    QCheckBox* checkBox = static_cast<QCheckBox*>(editor);
//    checkBox->setText(value);
}

void CheckBoxItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    QCheckBox* checkBox = static_cast<QCheckBox*>(editor);
        model->setData(index, "true", Qt::EditRole);
}

void CheckBoxItemDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    editor->setGeometry(option.rect);
}

//With help from https://www.qtcentre.org/threads/60997-Get-checkbox-of-QStandardItem 20190730
bool CheckBoxItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index)
{
    Q_UNUSED(option)
    if(event->type() == QEvent::MouseButtonRelease){
        QVariant value = index.data(Qt::CheckStateRole);
        Qt::CheckState state = (static_cast<Qt::CheckState>(value.toInt()) == Qt::Checked ? Qt::Unchecked: Qt::Checked);
        return model->setData(index,state, Qt::CheckStateRole);
    }

    return false;

}

//void CheckBoxItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
//{
//    painter->fillRect(option.rect, option.palette.highlight());
//    if(option.state == QStyle::State_On)
//        drawCheck(painter,option,option.rect, option.state());
//}

void CheckBoxItemDelegate::drawCheck(QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect, Qt::CheckState state) const
{
    if(state == Qt::Checked){
        painter->setPen(QColor(Qt::red));
        painter->drawLine(option.rect.topLeft(),option.rect.bottomRight());
        painter->drawLine(option.rect.topRight(),option.rect.bottomLeft());
        return;
    }
    painter->setPen(QColor(Qt::blue));
    painter->drawLine(option.rect.topLeft(),option.rect.bottomRight());
    painter->drawLine(option.rect.topRight(),option.rect.bottomLeft());
}
