//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "createprofiledialog.h"
#include "ui_createprofiledialog.h"
#include <QDebug>
#include <QMessageBox>
#include <QButtonGroup>

CreateProfileDialog::CreateProfileDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CreateProfileDialog)
{
    ui->setupUi(this);
    QObject::connect(ui->buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
    QObject::connect(ui->buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
    QObject::connect(this, &CreateProfileDialog::accepted, this, &CreateProfileDialog::sendAddCrossSectionSignal);
}

CreateProfileDialog::~CreateProfileDialog()
{
    delete ui;
}

void CreateProfileDialog::setPointModel(PointsModel * pointModel)
{
    mPointModel = new PointDescriptionModel(pointModel);
    ui->pointDescriptionSelect->setModel(mPointModel);
}

void CreateProfileDialog::setAlignmentModel(QAbstractItemModel *alignmentModel)
{
    mAlignmentModel = alignmentModel;
    ui->alignmentSelect->setModel(mAlignmentModel);
    ui->alignmentSelect->setModelColumn(1);
}

void CreateProfileDialog::sendAddCrossSectionSignal()
{
    //If no alignment is selected just exit.
    QString pointDescriptions = "";
    if(ui->alignmentSelect->selectionModel()->selectedIndexes().size() == 0){
        QMessageBox msgBox;
        msgBox.setText("No Alignment Selected");
        msgBox.exec();
        return;
    }
    QGuiApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    QModelIndex alignmentIndex = ui->alignmentSelect->selectionModel()->selectedIndexes().at(0);
    if (ui->selectAllCheck->checkState() == Qt::Unchecked){
        if(ui->pointDescriptionSelect->selectionModel()->selectedIndexes().size()>0){
            QModelIndexList pointsIndexList = ui->pointDescriptionSelect->selectionModel()->selectedIndexes();
            for(int i = 0; i < pointsIndexList.length()-1; i++){
                pointDescriptions += mPointModel->data(pointsIndexList.at(i)).toString();
                pointDescriptions += ",";
            }
            pointDescriptions += mPointModel->data(pointsIndexList.at(pointsIndexList.length()-1)).toString();
        }
    }
    qDebug() << "CreateProfileDialog::sendAddCrossSectionSignal pointDescriptions " << pointDescriptions;
    if(ui->mLongProfileRadioButton->isChecked()){
        emit addProfile(alignmentIndex, pointDescriptions, ui->mTolerance->value(),1);

    }
    else {
        emit addProfile(alignmentIndex, pointDescriptions, ui->mTolerance->value(),0);
    }
    QGuiApplication::setOverrideCursor(QCursor(Qt::ArrowCursor));
}
