
//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "gatdataexport.h"
#include <QDebug>


GATDataExport::GATDataExport()
{
    mFile = new QFile();
}

GATDataExport::GATDataExport(QString fileName)
{
    mFile = new QFile();
    setFileName(fileName);
}

GATDataExport::~GATDataExport()
{
    mFile->close();
    delete mFile;
}

bool GATDataExport::setFileName(QString fileName)
{
    //TODO: check to make sure file doesn't exist
    mFile->setFileName(fileName);
    if(mFile->exists()){
        mFile->setFileName("");
        return false;
    }
    else {
        return mFile->open(QFile::WriteOnly);
    }
}

bool GATDataExport::exportFile(const int mode)
{
    switch (mode) {
    case CSV:
        return exportCSV();
    case XML:
        return exportXML();
    default:
        return false;

    }
}

void GATDataExport::setModel(QAbstractItemModel *model)
{
    mModel = model;
}

void GATDataExport::closeFile()
{
        mFile->close();
}

bool GATDataExport::exportCSV()
/*! \brief Export cross section model. */
{

    if(!mModel){
        return false;
    }

    QTextStream textStream(mFile);
    exportHeader(textStream);
    for(int i = 0; i < mModel->rowCount(); i++){
        for(int j = 0; j < mModel->columnCount(); j++){
            QModelIndex index = mModel->index(i,j);
            textStream << mModel->data(index,Qt::DisplayRole).toString();
            textStream << ", ";
        }
        textStream << "\n";
        QModelIndex parent = mModel->index(i,0);
        bool exportHeaderCheck = false;
        if(mModel->hasChildren(parent)){
            textStream << "\n";
            exportCSVTable(textStream, parent);
            exportHeaderCheck = true;
        }
        parent = mModel->index(i,1);
        if(mModel->hasChildren(parent)){
            textStream  << "\n";
            exportCSVTable(textStream, parent);
            exportHeaderCheck = true;
        }
        if(exportHeaderCheck){
            exportHeader(textStream);
        }
    }
    qDebug() << "GATDataExport::exportCSV finished data read about to close";

    return true;
}
void GATDataExport::exportHeader(QTextStream &textStream){
    for(int i = 0 ; i < mModel->columnCount(); i++){
        qDebug() << "GATDataExport::exportCSV first for loop";
        textStream << mModel->headerData(i, Qt::Horizontal, Qt::DisplayRole).toString();
        textStream << ", ";
    }
    textStream << "\n";
}

void GATDataExport::exportCSVTable(QTextStream &textStream, QModelIndex &parent){

    for(int i = 0 ; i < mModel->columnCount(parent); i++){
        if(parent.column()==0){
            textStream << mModel->headerData(i+100, Qt::Horizontal, Qt::DisplayRole).toString();
        textStream << ", ";
        }
        if(parent.column()==1){
            textStream << mModel->headerData(i+200, Qt::Horizontal, Qt::DisplayRole).toString();
        textStream << ", ";
        }
    }
    textStream << "\n";
    for(int i = 0; i < mModel->rowCount(parent); i++){
        for(int j = 0; j < mModel->columnCount(parent); j++){
            QModelIndex index = mModel->index(i,j,parent);
            textStream << mModel->data(index, Qt::DisplayRole).toString();
            textStream << ", ";
        }
        textStream << "\n";
    }
    textStream << "\n";
}

bool GATDataExport::exportXML()
{
    //TODO: write this
    return false;
}
