//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef GATPLANVIEW_H
#define GATPLANVIEW_H

#include <QObject>
#include <QGraphicsView>
#include "gatplanviewscene.h"
#include "gatscalebar.h"
#include <QGestureEvent>

class GATPlanView : public QGraphicsView
{
    Q_OBJECT
public:
    GATPlanView(QWidget* parent = nullptr);
    void setPanMode();
    void setToolMode(const GATPlanViewScene::GATSceneMode mode);
    void setScaleBar(GATScaleBar*);
    void setScene(GATPlanViewScene *);

public slots:
    void zoomExtents();
    void zoomIn();
    void zoomOut();

protected Q_SLOTS:
    void wheelEvent(QWheelEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    bool viewportEvent(QEvent *event) override;
    bool event(QEvent *event) override;

private:
    QPointF mMousePressCoordinate;
    bool mPan;
    GATScaleBar* mScaleBar;
    qreal totalScaleFactor;
    GATPlanViewScene* mScene;
    GATPlanViewScene::GATSceneMode mMode;
    bool processGesture(QGestureEvent *event);
    void pinchTriggered(QPinchGesture *);
};

#endif // GATPLANVIEW_H
