//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "gatplanviewscene.h"
//#include "pointsmodel.h"
//#include <QModelIndex>
#include <QDebug>
#include <memory>
#include <QRectF>
#include "alignmentlayer.h"
#include "pointlayer.h"


GATPlanViewScene::GATPlanViewScene(QObject *parent) : QGraphicsScene (parent)
{
    //mBackgroundImage = new BackgroundImage();
    //qDebug() << "GATPlanViewScene::GATPlanViewScene " << mBackgroundImage->getInsertPosition();
    currentMode = noMode;
    setItemIndexMethod(QGraphicsScene::ItemIndexMethod::NoIndex);
    //this->addItem(mBackgroundImage);
}

GATPlanViewScene::~GATPlanViewScene()
{

}

void GATPlanViewScene::setScale(qreal scale)
{
    mScale = scale;
}

void GATPlanViewScene::setScaleBar(GATScaleBar *scaleBar)
{
    mScaleBar = scaleBar;
    addItem(mScaleBar);
    qDebug() << "GATPLanViewScene::setScaleBar";
}

QRectF GATPlanViewScene::itemsBoundingRect()
{
    //qDebug() << "GATPlanViewScene::itemsBoundingRect()";
    if(items().isEmpty())
    {
        return QRectF();
    }
    //Initialize the bounding rect.
    QRectF rect = mPointsGroup->boundingRect();
    qDebug() << "GATPlanViewScene::itemsBoundingRect mPointGroup" << rect;
    rect = rect.united(mAlignmentGroup->boundingRect());
    qDebug() << "GATPlanViewScene::itemsBoundingRect rect" << rect;

    QRectF sceneRect(0,0,rect.width()*10, rect.height()*10);
    sceneRect.moveCenter(rect.center());
    qDebug() << "GATPlanViewScene::itemsBoundingRect sceneRect" << sceneRect;
    setSceneRect(sceneRect);
    return rect;
}


void GATPlanViewScene::setAlignmentPointsGroup(AlignmentLayer *alignmentPointsGroup)
{
    mAlignmentGroup = alignmentPointsGroup;
    this->addItem(mAlignmentGroup);

}

void GATPlanViewScene::setPointsGroup(PointLayer *pointsGroup)
{
    mPointsGroup = pointsGroup;
    this->addItem(mPointsGroup);
}

void GATPlanViewScene::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    qDebug() << "GATPlanViewScene::mousePressEvent current mode is: " << currentMode;
    if(event->button()==Qt::LeftButton){
        if(currentMode == addAlignmentPointMode){
            emit addAlignmentPoint(event->buttonDownScenePos(Qt::LeftButton).ry(), event->buttonDownScenePos(Qt::LeftButton).rx());
            event->accept();
        }
    }
    if(event->button()==Qt::RightButton){
        if(currentMode == addAlignmentPointMode){
            emit createCrossSection();
            event->accept();
        }
    }
    QGraphicsScene::mousePressEvent(event);
}

void GATPlanViewScene::setCurrentAlignment(const int cA)
{
    mSelectedAlignment = cA;
}

void GATPlanViewScene::setMode(const GATSceneMode mode)
{
    qDebug() << "GATPlanViewScene::setMode setting mode" << mode;
    currentMode = mode;
    if(mode == GATPlanViewScene::moveAlignmentPointMode)
    {
        //emit setPointMode(GATPlanViewScene::moveAlignmentPointMode);
        mAlignmentGroup->setCurrentMode(GATPlanViewScene::moveAlignmentPointMode);
        return;
    }
    if(mode == GATPlanViewScene::deleteAlignmentPointMode){
        //emit setPointMode(mode);
        mAlignmentGroup->setCurrentMode(GATPlanViewScene::deleteAlignmentPointMode);
        return;
    }
    if(mode == GATPlanViewScene::insertAlignmentPointMode){
        //emit setPointMode(mode);
        mAlignmentGroup->setCurrentMode(GATPlanViewScene::insertAlignmentPointMode);
    }
    mAlignmentGroup->setCurrentMode(mode);
    //emit setPointMode(mode);
}


void GATPlanViewScene::setBackgroundImage(QString fileName)
{
    Q_UNUSED(fileName)
  //  if(!mBackgroundImage->scene()){
  //      this->addItem(mBackgroundImage);
   // }
    //mBackgroundImage->setBackgroundImage(fileName);
}

void GATPlanViewScene::recalculateSceneRect()
{
    QRectF mBoundingRect = this->itemsBoundingRect();
    QRectF  mSceneRect;
    qDebug() << "GATPlanViewScene recalculatesceneRect boundingRect topleft, bottomRight" << mBoundingRect.topLeft() << ", " << mBoundingRect.bottomRight();
    QPointF mTopLeft(mBoundingRect.topLeft().x()-mBoundingRect.size().width()/2,
                        mBoundingRect.topLeft().y()-mBoundingRect.size().height()/2);
    //    QPointF mBottomRight(QGraphicsScene::sceneRect().topLeft().x()+QGraphicsScene::sceneRect().size().width(),
    //                         QGraphicsScene::sceneRect().topLeft().y()-QGraphicsScene::sceneRect().size().height());
    mSceneRect.setTopLeft(mTopLeft);
    mSceneRect.setSize(QSizeF(mBoundingRect.size().width()*2,mBoundingRect.size().height()*2));
    qDebug() << "GATPlanViewScene::sceneRect graphicsView sceneRect " << QGraphicsScene::sceneRect() << " calculated " << mSceneRect;
    setSceneRect(mSceneRect);
}


