//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef GATPLANVIEWSCENE_H
#define GATPLANVIEWSCENE_H

#include <QVector>
#include <vector>
#include <QtWidgets\QGraphicsScene>
#include <QGraphicsSceneMouseEvent>


#include "point.h"
#include "pointsmodel.h"
#include "alignmentmodel.h"
//#include "backgroundimage.h"
#include <QFile>
#include "gatscalebar.h"


class AlignmentLayer;
class PointLayer;


class GATPlanViewScene : public QGraphicsScene
{
        Q_OBJECT
public:
    enum GATSceneMode{
        noMode = 0,
        addPointMode,
        addAlignmentPointMode,
        insertAlignmentPointMode,
        panMode,
        zoomOutMode,
        zoomInMode,
        deleteAlignmentPointMode,
        moveAlignmentPointMode
    };

    GATPlanViewScene(QObject* parent =nullptr);
    ~GATPlanViewScene();
    void addBackground(const QFile &);
    void setScale(qreal scale);
    void setScaleBar(GATScaleBar *);
    QRectF itemsBoundingRect();
    void setAlignmentPointsGroup(AlignmentLayer * alignmentPointsGroup);
    void setPointsGroup(PointLayer * pointsGroup);

protected Q_SLOTS:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    //void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    //void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

signals:
    //void addPoint(const QPointF) const;
    void addAlignmentPoint(const double, const double) const;
    void insertAlignmentPoint(const double, const double, const int) const;
    void createCrossSection();
    //void setPointMode(const int) const;

public slots:
    //void rowsInsertedPoints(const QModelIndex &index, const int first, const int last);
    //void rowsRemovedPoints(const QModelIndex &index, const int first, const int last);
    //void rowsInsertedAlignments(const QModelIndex &index, const int first, const int last);
    //void rowsRemovedAlignments(const QModelIndex &index, const int first, const int last);
    //void dataUpdatedAlignments(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    void setCurrentAlignment(const int);
    void setMode(const GATPlanViewScene::GATSceneMode);
    //void modelAboutToBeResetPoints();
    //void modelAboutToBeResetAlignments();
    //void modelResetPoints();
    //void modelResetAlignments();
    void setBackgroundImage(QString fileName);
    //void printItems();

private:
    //PointsModel * mPointsModel;
    //AlignmentModel *mAlignmentModel;
    qreal m_width;
    qreal m_height;
    int mSelectedAlignment;
    int mSelectedAlignmentSegment;
    int currentMode;
    qreal mScale;
    void recalculateSceneRect();
//    BackgroundImage* mBackgroundImage;

    PointLayer* mPointsGroup;
    AlignmentLayer* mAlignmentGroup;
    GATScaleBar* mScaleBar;
};

#endif // GATPLANVIEWSCENE_H
