#include "gatscalebar.h"
#include <QPainter>
#include <QDebug>
#include <QWidget>
#include <QGraphicsView>
#include <QtMath>

GATScaleBar::GATScaleBar(QGraphicsItem* parent):QGraphicsItem(parent)
{
    mView = new QGraphicsView();
    setFlag(QGraphicsItem::ItemSendsGeometryChanges);

}

void GATScaleBar::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    int scaleBarLength = 100;
    int scaleBarMargin = 20;
    //set topLeft based on the view
    topLeft = mView->mapToScene(QPoint(scaleBarMargin,scaleBarMargin));
    QPointF maxSize = mView->mapToScene(QPoint(scaleBarLength,0));
    //qDebug() << "GATScaleBar::paint topLeft maxSize " << topLeft << " " << maxSize;

    //This should truncate from the float to the int
    int power = qFloor(qLn(maxSize.x()-topLeft.x())/qLn(10));
    if(power < 0){
        power = 0; //Min size is 1
    }
    int scaleBarSize = qPow(10,power);
    //this is the overall transform
    QPen pen;
    pen.setCosmetic(true);
    painter->setPen(pen);

    //Draw the line
    QLineF scaleBarLine = QLineF(topLeft, QPointF(topLeft.x()+scaleBarSize,topLeft.y()));
    painter->drawLine(scaleBarLine);
    //the line should be 10 units long.
    bottomRight =  QPointF(topLeft.x()+scaleBarSize+10, topLeft.y() + 20);

    //Draw the text in view scale.
    double scaleValue = scale()/painter->transform().m11();
    painter->save();
    QFont scaleBarFont;
    //scale painter to have a scale of 1 and flip in y direction so text is the correct direciton
    painter->scale(scaleValue,-1*scaleValue);

    scaleBarFont.setPointSize(10);
    painter->setFont(scaleBarFont);
    painter->drawText(QPointF(topLeft.x()/scaleValue,-1*topLeft.y()/scaleValue),QString::number(scaleBarSize,'f',0));

    //put the painter back to stored state to continue drawing.
    painter->restore();

    //qDebug() << "GATScaleBar::paint power " << power;
    //qDebug() << "scaleBarSize " << scaleBarSize;
}

QRectF GATScaleBar::boundingRect() const
{
    return QRectF(topLeft, bottomRight);
}

void GATScaleBar::setPosition(const QPointF position){
    topLeft = position;
}

void GATScaleBar::setView(QGraphicsView *view)
{
    mView = view;
}

void GATScaleBar::updatePosition()
{
    prepareGeometryChange();
    bottomRight = mView->mapToScene(QPoint(110,20));
    topLeft = mView->mapToScene(QPoint(10,10));
    update();


}
