//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include <QKeyEvent>
#include <QDebug>

#include "gattableview.h"


GATTableView::GATTableView(QWidget* parent) : QTableView(parent)
{
    this->setEditTriggers(QAbstractItemView::AnyKeyPressed | QAbstractItemView::SelectedClicked);
}

void GATTableView::keyReleaseEvent(QKeyEvent *event)
{

    if(event->key() == Qt::Key_Delete ){
        this->setCursor(Qt::WaitCursor);

        //https://www.qtcentre.org/threads/4885-Remove-selected-rows-from-a-QTableView
        QItemSelection selection( this->selectionModel()->selection() );

        QList<int> rows;
        foreach( const QModelIndex & index, selection.indexes() ) {
           rows.append( index.row() );
        }

        std::sort(rows.begin(),rows.end() );

        int prev = -1;
        for( int i = rows.count() - 1; i >= 0; i -= 1 ) {
           int current = rows[i];
           if( current != prev ) {
              this->model()->removeRows( current, 1 );
              prev = current;
           }
        }
        this->setCursor(Qt::ArrowCursor);
    }
    else {
        QTableView::keyReleaseEvent(event);
    }
}

//Modified from: https://pohlondrej.com/qtableview-excel-like-editing/
void GATTableView::keyPressEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Return){
        qint32 nNextRow = currentIndex().row()+1; //this should be updated to move right then over
        if(nNextRow+1 > model()->rowCount(currentIndex()))
        {
            //put it back if this would take us past the end
            nNextRow = nNextRow-1;
        }
        if(state() == QAbstractItemView::EditingState)
        {
            //if editing
            QModelIndex oNextIndex = model()->index(nNextRow, currentIndex().column());
            setCurrentIndex(oNextIndex);
            selectionModel()->select(oNextIndex, QItemSelectionModel::ClearAndSelect);
        }
        else {
            {
                edit(currentIndex());
            }
        }
    }
    else{
        //any other key.
        QAbstractItemView::keyPressEvent(event);
    }
}
