//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtWidgets/QMainWindow>
#include <QLayout>
#include "gatdatamanagerqmlplugin.h"
#include "ui_mainwindow.h"
#include "alignmentpointsmodel.h"
#include "profilepointsmodel.h"
#include "hydraulicparametersmodel.h"
#include "graphicsviewwindow.h"
#include "profilechartwidget.h"
#include "gatplanview.h"


#include "checkboxitemdelegate.h"

class PointsWidget;
class QGraphicsView;
class GATPlanViewScene;
class PointsModel;
class AlignmentModel;
class ProfileModel;
class MeasurementPointModel;
class GATPlanView;
class ProfileChartWidget;
class GATScaleBar;
class AlignmentLayer;
class PointLayer;
class MeasurementItemDelegate;
class ProfileChartWidget;

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow() override;
    void openFile();
    void newFile();

    //void setCurrentCrossSection(const QModelIndex &);
    //void setCurrentLongProfile(const QModelIndex &);
    void closeEvent(QCloseEvent *event) override;
    void resetTabs();

private:
    void readSettings();
    void writeSettings();
    void restore();
    Ui::MainWindow *ui;
    GATDataManagerQMLPlugin *mDataManager;
    GATPlanViewScene* mGATPlanViewScene;
    bool mMobileMode;

    ProfileChartWidget* mLongProfileChartWidget;
    GATScaleBar* mScaleBar; //need to connect this to the view

    //Points Models
    PointsModel* mPointsModel;

    //Alignment Models
    AlignmentModel * mAlignmentModel;
    AlignmentPointsModel *mAlignmentPointsModel;

    //Cross Section Models
    ProfileModel *mCrossSectionModel;
    ProfilePointsModel *mCrossSectionPointsModel;
    MeasurementPointModel *mCrossSectionMeasurementsModel;
    MeasurementPointModel *mAlignmentMeasurementModel;
    MeasurementPointModel *mLongProfileMeasurementModel;
    HydraulicParametersModel *mHydraulicParametersModel;

    //Longitudinal Profile Models
    ProfileModel *mLongProfileModel;
    ProfilePointsModel *mLongProfilePointsModel;

    //Alignment, Cross-Section, and Profile Widgets
    QDockWidget *mAlignmentPointsDockWidget;
    QDockWidget *mAlignmentMeasurementDockWidget;

    MeasurementItemDelegate* alignmentMeasurementTypeDelegate;
    GATTableView *alignmentPointsTable;
    GATTableView *alignmentMeasurementTable;

    QModelIndex mContextIndex;
    //void loadStyleSheet();
    void connectSignals();
    void setupModels();
    void setupObjects();
    void loadFileDialog();
    void setFile(QString fileName);

    AlignmentLayer* mAlignmentLayer;
    PointLayer* mPointLayer;

public slots:
    //void setAddPointMode();
    void setAddAlignmentPointMode();
    void setMoveAlignmentPointMode();
    void setDeleteAlignmentPointMode();
    void setInsertAlignmentPointMode();

    void getNEZDFileImport();
    void clearGraphicsViewMode();
    void calculateProfile();
    //void calculateHydraulicParametersDialog();
    void addAlignment();
    void addAlignmentPoint(const double, const double);
    void importCrossSection();
    void showAbout();
    void mobileView();
    void readLandXML();
    void addProfile(const QModelIndex&,
                    const QString pointFilters,
                    const double tolerance,
                   const int profileType);

    void alignmentContextMenu(QPoint pos);
    void setBackgroundImage();
    void exportTableData();

    void setCurrentAlignment(const QModelIndex &);
    void setCurrentAlignmentRow(const int row);

private slots:
    //Open Dock Widget
    void openAlignmentPointsDock();
    void openAlignmentMeasurementDock();

    void showEvent(QShowEvent* event) override;
};

#endif // MAINWINDOW_H
