#include "pointgraphicsitem.h"
#include <QPainter>

PointGraphicsItem::PointGraphicsItem(QGraphicsItem* parent) : QGraphicsItem(parent),
    mCoordinateTransform(GATCoordinateTransform::instance())

{
    mRadius = 5;
}

void PointGraphicsItem::setPosition(double easting, double northing)
{
    mEasting = easting;
    mNorthing = northing;
    this->setPos();
}

void PointGraphicsItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(widget)
    Q_UNUSED(option)
    double hScale = painter->transform().m11();
    mRadius = 5/hScale;
    QBrush brush(Qt::red);
    QPen pen(Qt::blue,1/hScale);
    painter->setBrush(brush);
    painter->setPen(pen);
    painter->drawEllipse(this->boundingRect());
}

QRectF PointGraphicsItem::boundingRect() const
{
    return QRectF(-mRadius, -mRadius, width(), height());
}

qreal PointGraphicsItem::width() const
{
    return 2*mRadius;
}

qreal PointGraphicsItem::height() const
{
    return 2*mRadius;
}

void PointGraphicsItem::setPos()
{
    QPointF point = mCoordinateTransform.toPlottingCoordinates(QPointF(mEasting, mNorthing));
    QGraphicsItem::setPos(point);
}


