//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#include "pointswidget.h"
#include <QtWidgets/QFormLayout>
#include <QtCore>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QFileDialog>
#include <QUrl>

#include "pointsmodel.h"

PointsWidget::PointsWidget(QWidget *parent) :
    QWidget(parent)
{
    mPointsModel = nullptr;
    setupUi(this);
}

void PointsWidget::setModel(PointsModel * pm){
    mPointsModel = pm;
    mPointsTable->setModel(mPointsModel);
    setupPointsList();
}

void PointsWidget::setSelectionModel(QItemSelectionModel * selectionModel){
    mPointsTable->setSelectionModel(selectionModel);
}

void PointsWidget::setupUi(QWidget *pw)
{
    qDebug() << "PointsWidget::setupUi enter";
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setMargin(5);
    mPointsTable = new QTableView(pw);

    setupPointsList();
    //mPointsList->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
    addPointButton = new QPushButton(pw);
    importPointButton = new QPushButton(pw);

    addPointButton->setText("Add Point");

    importPointButton->setText("Import PNEZD File");
    layout->addWidget(mPointsTable);
    layout->addWidget(addPointButton);
    layout->addWidget(importPointButton);
    pw->setLayout(layout);

}

void PointsWidget::connectSignals()
{
    QObject::connect(addPointButton, &QPushButton::pressed,this,&PointsWidget::addPointDialog);
    QObject::connect(this, &PointsWidget::databaseUpdated, mPointsModel, &PointsModel::databaseUpdated);
    QObject::connect(importPointButton, &QPushButton::pressed, this, &PointsWidget::getNEZDFileImport);
}

void PointsWidget::addPointDialog() const
{
    qDebug() << "About to emit button pressed";
    QDialog* pointInputDialog = new QDialog();
    pointInputDialog->setModal(true);

    QFormLayout* dialogLayout = new QFormLayout();
    dialogLayout->setMargin(20);

    //Add the inputs
    QLineEdit* northingInput = new QLineEdit();
    QLineEdit* eastingInput = new QLineEdit();
    QLineEdit* elevationInput=  new QLineEdit();
    QLineEdit* descriptionInput = new QLineEdit();
    dialogLayout->addRow("Northing", northingInput);
    dialogLayout->addRow("Easting", eastingInput);
    dialogLayout->addRow("Elevation", elevationInput);
    dialogLayout->addRow("Description", descriptionInput);

    QPushButton* okButton = new QPushButton(pointInputDialog);
    okButton->setText("Submit");
    dialogLayout->addRow(okButton);

    QPushButton* cancelButton = new QPushButton(pointInputDialog);
    cancelButton->setText("Cancel");
    dialogLayout->addRow(cancelButton);

    pointInputDialog->setLayout(dialogLayout);

    connect(cancelButton,&QPushButton::released,pointInputDialog,&QDialog::reject);
    connect(okButton, &QPushButton::released,pointInputDialog,&QDialog::accept);
    pointInputDialog->exec();

    if(pointInputDialog->result()){
        emit addPoint(northingInput->text().toDouble(),eastingInput->text().toDouble(),elevationInput->text().toDouble(),descriptionInput->text());
    }
}


void PointsWidget::setupPointsList(){
    mPointsTable->setSelectionMode(QAbstractItemView::SingleSelection);
    mPointsTable->setEditTriggers(QAbstractItemView::AllEditTriggers);
}

void PointsWidget::databaseUpdatedSlot() const
{
    emit databaseUpdated();
}

void PointsWidget::getNEZDFileImport()
{
    QSettings settings;
    const QString DEFAULT_DIR_KEY("default_dir");
    QString fileName = QFileDialog::getOpenFileName(this,
                                                tr("Open PNEZD File"),
                                                settings.value(DEFAULT_DIR_KEY).toString(),
                                                tr("csv (*.csv);; txt (*.txt)"));
    if(!fileName.isEmpty()){
        mPointsModel->addPointsFromFile(",",fileName);
    }
}


