//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef CROSSSECTIONCHARTVIEW_H
#define CROSSSECTIONCHARTVIEW_H
//event handeling from: https://doc.qt.io/qt-5/qtcharts-zoomlinechart-example.html
//tool tips: https://doc.qt.io/qt-5/qtcharts-callout-view-h.html
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QRubberBand>
#include <QtCharts/QChartGlobal>

class QGraphicsScene;
class QMouseEvent;
class QResizeEvent;

QT_CHARTS_BEGIN_NAMESPACE
class QChart;
QT_CHARTS_END_NAMESPACE

QT_CHARTS_USE_NAMESPACE

class GATRubberBand;
class ProfileChartView: public QGraphicsView
{
    Q_OBJECT
public:

    enum crossSectionChartRole {
        pan = 0,
        addStage,
        measure
    };

    ProfileChartView(QChart *chart, QWidget *parent = nullptr);

signals:
    void addHydraulicStage(double stage);
    void statusMessage(const QString);
    void addMeasurement(const QPointF pointOne, const QPointF pointTwo);

public slots:
    void setMode(int mode);
    void resizeEvent(QResizeEvent *event) override;

protected:
    bool viewportEvent(QEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void keyPressEvent(QKeyEvent *event) override;
    void wheelEvent(QWheelEvent *wheelEvent) override;

private:
    bool mIsTouching;
    int mMode;
    bool mPan;
    int mMouseDownX;
    int mMouseDownY;
    QPointF mMeasurePointOne;
    QPointF mMeasurePointTwo;
    QChart *mChart;
    QGraphicsSimpleTextItem *mToolTip;
    GATRubberBand *mRubberBand;
    bool mMeasuring;
    QPoint origin;
};

#endif // CROSSSECTIONCHARTVIEW_H
