//Geomorphic Assessment Tools (GAT) for processing geomorphic survey data
//Copyright (C) 2019 Tailwater Limited, Andrea Taillacq

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef PROFILECHARTWIDGET_H
#define PROFILECHARTWIDGET_H

#include <QtCharts>
#include <QLineSeries>
#include <QPersistentModelIndex>
#include "profilepointsmodel.h"
#include "profilechartview.h"
#include <QComboBox>
#include "gattableview.h"
#include "hydraulicparametersmodel.h"
#include "measurementpointmodel.h"
#include "measurementitemdelegate.h"

class ProfileChartWidget: public QWidget
{
    Q_OBJECT
public:
    ProfileChartWidget(bool XS = true, QWidget *parent=nullptr);
    void setModel(ProfileModel*);
    void connectSignals();
    void setRootIndex(const QModelIndex &index);

public slots:
    void resizeEvent(QResizeEvent *event);
    void addHydraulicStage(double stage);
    void addMeasurement(const QPointF, const QPointF);
    void setPanMode();
    void setAddStageMode();
    void setMeasureMode();
    void updateStatusBar(const QString);
    void setRow(int row);
    void resetChart();
    void editProfile();
    void rowsInserted(const QModelIndex &parent, int first, int last);
    void rowsRemoved(const QModelIndex &parent, int first, int last);

private:
    //QList<QLineSeries *> mCrossSectionLineSeries;
    QVXYModelMapper *mPointMapper;
    QLineSeries* mSelectedProfileLineSeries;
    ProfilePointsModel* mSelectedProfileSeriesModel;

    //Lists to hold the selected profiles hydraulic parameters and measurements
    QHash<QPersistentModelIndex, QLineSeries *> mHydraulicParameterSeries;
    QList<QLineSeries *> mMeasurementSeries;

    //QList<ProfilePointsModel *> mCrossSectionSeriesModels;
    ProfileModel* mModel;
    QChart*  mChart;
    QValueAxis *mXAxis;
    QValueAxis *mYAxis;
    ProfileChartView* mChartView;

    QVBoxLayout* mLayout;
    //void setupSeries();
    void setupUi();
    void loadSelectedSeries();
    void loadMeasurementSeries();
    QModelIndex mSelectedProfileIndex;

    QActionGroup* mActionChartTools;
    QAction* mActionAddStage;
    QAction* mActionPan;
    QAction* mActionMeasure;
    QAction* mActionEditProfile;
    QAction* mActionCenterChart;
    QToolBar* mToolBar;
    QStatusBar* mStatusBar;

    QComboBox* mProfileSelectList;

    HydraulicParametersModel* mHydraulicParametersModel;
    MeasurementPointModel* mMeasurementPointModel;

    GATTableView* mProfilePointsTable;
    GATTableView* mHydraulicParametersTable;
    GATTableView* mMeasurementsTable;
    QSplitter *mChartTableSplitter;
    QTabWidget *mTableTabs;
    MeasurementItemDelegate *mMeasurementTypeDelegate;

    bool hasHydraulicParameters;
};

#endif // PROFILECHARTWIDGET_H
